'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var reactQuery = require('react-query');
var jotai = require('jotai');

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

var queryClientAtom = jotai.atom(null);

var getQueryClient = function getQueryClient(get, set) {
  var queryClient = get(queryClientAtom);

  if (queryClient === null) {
    queryClient = new reactQuery.QueryClient();
    set(queryClientAtom, queryClient);
  }

  return queryClient;
};

var createPending = function createPending() {
  var pending = {
    fulfilled: false
  };
  pending.promise = new Promise(function (resolve) {
    pending.resolve = function (data) {
      resolve(data);
      pending.fulfilled = true;
    };
  });
  return pending;
};

function atomWithQuery(createQuery) {
  var pendingAtom = jotai.atom(createPending());
  var dataAtom = jotai.atom(null);
  var queryAtom = jotai.atom(function (get) {
    var options = typeof createQuery === 'function' ? createQuery(get) : createQuery;
    var observerAtom = jotai.atom(null, function (get, set, action) {
      if (action.type === 'init') {
        var pending = get(pendingAtom);

        if (pending.fulfilled) {
          set(pendingAtom, createPending()); // new fetch
        }

        action.intializer(getQueryClient(get, set));
      } else if (action.type === 'data') {
        set(dataAtom, action.data);

        var _pending = get(pendingAtom);

        if (!_pending.fulfilled) {
          _pending.resolve(action.data);
        }
      }
    });

    observerAtom.onMount = function (dispatch) {
      var unsub;

      var intializer = function intializer(queryClient) {
        var observer = new reactQuery.QueryObserver(queryClient, options);
        observer.subscribe(function (result) {
          // TODO error handling
          if (result.data !== undefined) {
            dispatch({
              type: 'data',
              data: result.data
            });
          }
        });

        if (unsub === false) {
          observer.destroy();
        } else {
          unsub = function unsub() {
            observer.destroy();
          };
        }
      };

      dispatch({
        type: 'init',
        intializer: intializer
      });
      return function () {
        if (unsub) {
          unsub();
        }

        unsub = false;
      };
    };

    return [options, observerAtom];
  }, /*#__PURE__*/function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(get, set, action) {
      var _getQueryClient$getQu, _get2, options;

      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (!(action.type === 'refetch')) {
                _context.next = 6;
                break;
              }

              _get2 = get(queryAtom), options = _get2[0];
              set(pendingAtom, createPending()); // reset pending

              (_getQueryClient$getQu = getQueryClient(get, set).getQueryCache().find(options.queryKey)) == null ? void 0 : _getQueryClient$getQu.reset();
              _context.next = 6;
              return getQueryClient(get, set).refetchQueries(options.queryKey);

            case 6:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function (_x, _x2, _x3) {
      return _ref.apply(this, arguments);
    };
  }());
  var queryDataAtom = jotai.atom(function (get) {
    var _get3 = get(queryAtom),
        observerAtom = _get3[1];

    get(observerAtom); // use it here

    var data = get(dataAtom);
    var pending = get(pendingAtom);

    if (!pending.fulfilled) {
      return pending.promise;
    } // we are sure that data is not null


    return data;
  }, function (_get, set, action) {
    return set(queryAtom, action);
  } // delegate action
  );
  return queryDataAtom;
}

exports.atomWithQuery = atomWithQuery;
